//+------------------------------------------------------------------+
//Semi Standard Deviation Bands
//Indicator shows Moving Average and bands of semi standard deviation
//semi standard deviation caches separately deviation over Moving Average
//and below Moving average
//
//Parameters:
//MAPeriod        - Moving average Period
//BandsPeriod     - window used for calculating Standard deviation
//BandsShift      - shift for bands
//BandsDeviations - width of bands
//MovingDeviation - when false calculates deviation for whole period 
//                  assuming current value of moving average as mean
//                  when true deviation for every poind in deviation
//                  window is calculated by its distance from moving
//                  average
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Marek Marciniak."
#property link      "marciniak.marek@gmail.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_color2 LightSeaGreen
#property indicator_color3 LightSeaGreen
//---- indicator parameters
extern int    MAPeriod=20;
extern int    BandsPeriod=20;
extern int    BandsShift=0;
extern double BandsDeviations=2.0;
extern bool   MovingDeviation=false;
extern int    MAType=MODE_SMA;
extern string sMAType="SMA=0 EMA=1 SMMA=2 LWMA=3";
extern int    PriceType=PRICE_MEDIAN;
extern string sPrice_Type="CLOS=0 OPEN=1 HIGH=2 LOW=3 MEDI=4 TYPI=5 WEIG=6";
//---- buffers
double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexBuffer(0,MovingBuffer);
   SetIndexBuffer(1,UpperBuffer);
   SetIndexBuffer(2,LowerBuffer);
//----
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE);
//----
   SetIndexLabel(0,"MovingAverage");
   SetIndexLabel(1,"SemiUpperBand");
   SetIndexLabel(2,"SemiLowerBand");
//----
   SetIndexDrawBegin(0,MAPeriod+BandsPeriod+BandsShift);
   SetIndexDrawBegin(1,MAPeriod+BandsPeriod+BandsShift);
   SetIndexDrawBegin(2,MAPeriod+BandsPeriod+BandsShift);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Bollinger Bands                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int    i,k,counted_bars=IndicatorCounted();
   double deviation;
   double sumU,sumL,countU,countL,oldval,newres,hm,lm;
//----
   if(Bars<=BandsPeriod) return(0);
//---- initial zero
   if(counted_bars<1)
      for(i=1;i<=MAPeriod+BandsPeriod+BandsShift;i++){
        MovingBuffer[Bars-i]=EMPTY_VALUE;
        UpperBuffer[Bars-i]=EMPTY_VALUE;
        LowerBuffer[Bars-i]=EMPTY_VALUE;
      }
//----
   int limit=Bars-counted_bars;
   if(counted_bars>0) limit++;
   for(i=0; i<limit; i++)
     MovingBuffer[i]=iMA(NULL,0,MAPeriod,BandsShift,MAType,PriceType,i);
//----
   i=Bars-MAPeriod-BandsPeriod+1;
   if(counted_bars>MAPeriod+BandsPeriod-1) 
     i=Bars-counted_bars-1;
   while(i>=0){
     sumU=0.0;
     sumL=0.0;
     countU=0;
     countL=0;
     k=i+BandsPeriod-1;
     oldval=MovingBuffer[i];
     while(k>=i) {
       if(MovingDeviation){
         hm=High[k]-MovingBuffer[k];
         lm=Low[k]-MovingBuffer[k];
         if(hm>-lm) newres = hm; else newres = lm;
       }else
         newres=Close[k]-oldval;
           
       if (newres>0)
         sumU+=newres*newres; 
       
       if (newres<0)
         sumL+=newres*newres; 
       k--;
      } 
     deviation=BandsDeviations*MathSqrt(sumU/BandsPeriod);
     UpperBuffer[i]=oldval+deviation;
     deviation=BandsDeviations*MathSqrt(sumL/BandsPeriod);
     LowerBuffer[i]=oldval-deviation;
     i--;
   }
//----
   return(0);
  }
//+------------------------------------------------------------------+